@@ -111,7 +111,27 @@ class VFS(object):
         if rem:
             rp += "/" + rem
 
-        return fsdec(os.path.realpath(fsenc(rp)))
+        try:
+            return fsdec(os.path.realpath(fsenc(rp)))
+        except:
+            if not WINDOWS:
+                raise
+
+            # cpython bug introduced in 3.8, still exists in 3.9.1;
+            # some win7sp1 and win10:20H2 boxes cannot realpath a
+            # networked drive letter such as b"n:" or b"n:\\"
+            #
+            # requirements to trigger:
+            #  * bytestring (not unicode str)
+            #  * just the drive letter (subfolders are ok)
+            #  * networked drive (regular disks and vmhgfs are ok)
+            #  * on an enterprise network (idk, cannot repro with samba)
+            #
+            # hits the following exceptions in succession:
+            #  * access denied at L601: "path = _getfinalpathname(path)"
+            #  * "cant concat str to bytes" at L621: "return path + tail"
+            #
+            return os.path.realpath(rp)
 
     def ls(self, rem, uname, scandir, lstat=False):
         """return user-readable [fsdir,real,virt] items at vpath"""
@@ -168,8 +188,13 @@ class VFS(object):
         for vpath, apath, files, rd, vd in self.walk("", vrem, uname, dots, scandir):
             if flt:
                 files = [x for x in files if x[0] in flt]
-                rd = [x for x in rd if x[0] in flt]
-                vd = {x: y for x, y in vd.items() if x in flt}
+
+                rm = [x for x in rd if x[0] not in flt]
+                [rd.remove(x) for x in rm]
+
+                rm = [x for x in vd.keys() if x not in flt]
+                [vd.pop(x) for x in rm]
+
                 flt = None
 
             # print(repr([vpath, apath, [x[0] for x in files]]))
@@ -228,12 +253,6 @@ class AuthSrv(object):
     def log(self, msg, c=0):
         self.log_func("auth", msg, c)
 
-    def invert(self, orig):
-        if PY2:
-            return {v: k for k, v in orig.iteritems()}
-        else:
-            return {v: k for k, v in orig.items()}
-
     def laggy_iter(self, iterable):
         """returns [value,isFinalValue]"""
         it = iter(iterable)
@@ -490,7 +509,7 @@ class AuthSrv(object):
         with self.mutex:
             self.vfs = vfs
             self.user = user
-            self.iuser = self.invert(user)
+            self.iuser = {v: k for k, v in user.items()}
 
         # import pprint
         # pprint.pprint({"usr": user, "rd": mread, "wr": mwrite, "mnt": mount})
