@@ -75,7 +75,7 @@ makeSortable(ebi('files'), mp.read_order.bind(mp));
 var widget = (function () {
 	var ret = {};
 	var widget = ebi('widget');
-	var wtoggle = ebi('wtoggle');
+	var wtico = ebi('wtico');
 	var touchmode = false;
 	var side_open = false;
 	var was_paused = true;
@@ -113,14 +113,7 @@ var widget = (function () {
 
 		return false;
 	};
-	if (window.Touch) {
-		var touch_handler = function (e) {
-			touchmode = true;
-			return ret.toggle(e);
-		};
-		wtoggle.addEventListener('touchstart', touch_handler, false);
-	}
-	wtoggle.onclick = click_handler;
+	wtico.onclick = click_handler;
 	return ret;
 })();
 
@@ -321,10 +314,10 @@ function seek_au_sec(seek) {
 
 	mp.au.currentTime = seek;
 
+	// ogv.js breaks on .play() during playback
 	if (mp.au === mp.au_native)
-		// hack: ogv.js breaks on .play() during playback
 		mp.au.play();
-};
+}
 
 
 function song_skip(n) {
@@ -336,7 +329,7 @@ function song_skip(n) {
 		play(mp.order.indexOf(tid) + n);
 	else
 		play(mp.order[0]);
-};
+}
 
 
 // hook up the widget buttons
@@ -434,7 +427,7 @@ catch (ex) { }
 
 
 // plays the tid'th audio file on the page
-function play(tid, call_depth) {
+function play(tid, seek, call_depth) {
 	if (mp.order.length == 0)
 		return alert('no audio found wait what');
 
@@ -456,7 +449,7 @@ function play(tid, call_depth) {
 	}
 
 	// ogv.js breaks on .play() unless directly user-triggered
-	var hack_attempt_play = true;
+	var attempt_play = true;
 
 	var url = mp.tracks[tid];
 	if (need_ogv && /\.(ogg|opus)$/i.test(url)) {
@@ -465,7 +458,7 @@ function play(tid, call_depth) {
 		}
 		else if (window['OGVPlayer']) {
 			mp.au = mp.au_ogvjs = new OGVPlayer();
-			hack_attempt_play = false;
+			attempt_play = false;
 			mp.au.addEventListener('error', evau_error, true);
 			mp.au.addEventListener('progress', pbar.drawpos, false);
 			widget.open();
@@ -477,7 +470,7 @@ function play(tid, call_depth) {
 			show_modal('<h1>loading ogv.js</h1><h2>thanks apple</h2>');
 
 			import_js('/.cpr/deps/ogv.js', function () {
-				play(tid, 1);
+				play(tid, seek, 1);
 			});
 
 			return;
@@ -500,26 +493,31 @@ function play(tid, call_depth) {
 	setclass(oid, 'play act');
 	var trs = ebi('files').getElementsByTagName('tbody')[0].getElementsByTagName('tr');
 	for (var a = 0, aa = trs.length; a < aa; a++) {
-		trs[a].className = trs[a].className.replace(/ *play */, "");
+		clmod(trs[a], 'play');
 	}
 	ebi(oid).parentElement.parentElement.className += ' play';
 
 	try {
-		if (hack_attempt_play)
+		if (attempt_play)
 			mp.au.play();
 
 		if (mp.au.paused)
-			autoplay_blocked();
+			autoplay_blocked(seek);
+		else if (seek) {
+			seek_au_sec(seek);
+		}
 
-		var o = ebi(oid);
-		o.setAttribute('id', 'thx_js');
-		if (window.history && history.replaceState) {
-			hist_replace(document.location.pathname + '#' + oid);
+		if (!seek) {
+			var o = ebi(oid);
+			o.setAttribute('id', 'thx_js');
+			if (window.history && history.replaceState) {
+				hist_replace(document.location.pathname + '#' + oid);
+			}
+			else {
+				document.location.hash = oid;
+			}
+			o.setAttribute('id', oid);
 		}
-		else {
-			document.location.hash = oid;
-		}
-		o.setAttribute('id', oid);
 
 		pbar.drawbuf();
 		return true;
@@ -583,7 +581,7 @@ function unblocked() {
 
 
 // show ui to manually start playback of a linked song
-function autoplay_blocked() {
+function autoplay_blocked(seek) {
 	show_modal(
 		'<div id="blk_play"><a href="#" id="blk_go"></a></div>' +
 		'<div id="blk_abrt"><a href="#" id="blk_na">Cancel<br />(show file list)</a></div>');
@@ -599,6 +597,8 @@ function autoplay_blocked() {
 		if (e) e.preventDefault();
 		unblocked();
 		mp.au.play();
+		if (seek)
+			seek_au_sec(seek);
 	};
 	na.onclick = unblocked;
 }
@@ -607,8 +607,20 @@ function autoplay_blocked() {
 // autoplay linked track
 (function () {
 	var v = location.hash;
-	if (v && v.length == 12 && v.indexOf('#af-') === 0)
-		play(v.slice(2));
+	if (v && v.indexOf('#af-') === 0) {
+		var id = v.slice(2).split('&');
+		if (id[0].length != 10)
+			return;
+
+		if (id.length == 1)
+			return play(id[0]);
+
+		var m = /^[Tt=0]*([0-9]+[Mm:])?0*([0-9]+)[Ss]?$/.exec(id[1]);
+		if (!m)
+			return play(id[0]);
+
+		return play(id[0], parseInt(m[1] || 0) * 60 + parseInt(m[2] || 0));
+	}
 })();
 
 
@@ -625,13 +637,16 @@ function tree_neigh(n) {
 			break;
 		}
 	}
-	a += n;
-	if (a < 0)
-		a = links.length - 1;
-	if (a >= links.length)
-		a = 0;
+	if (act == -1)
+		return;
 
-	links[a].click();
+	act += n;
+	if (act < 0)
+		act = links.length - 1;
+	if (act >= links.length)
+		act = 0;
+
+	links[act].click();
 }
 
 
@@ -649,10 +664,13 @@ function tree_up() {
 
 
 document.onkeydown = function (e) {
-	if (document.activeElement != document.body && document.activeElement.nodeName.toLowerCase() != 'a')
+	if (!document.activeElement || document.activeElement != document.body && document.activeElement.nodeName.toLowerCase() != 'a')
 		return;
 
-	var k = e.code, pos = -1;
+	if (e.ctrlKey || e.altKey || e.shiftKey || e.metaKey || e.isComposing)
+		return;
+
+	var k = (e.code + ''), pos = -1;
 	if (k.indexOf('Digit') === 0)
 		pos = parseInt(k.slice(-1)) * 0.1;
 
@@ -772,6 +790,7 @@ document.onkeydown = function (e) {
 		// ebi('srch_q').textContent = JSON.stringify(params, null, 4);
 		var xhr = new XMLHttpRequest();
 		xhr.open('POST', '/?srch', true);
+		xhr.setRequestHeader('Content-Type', 'text/plain');
 		xhr.onreadystatechange = xhr_search_results;
 		xhr.ts = new Date().getTime();
 		xhr.send(JSON.stringify(params));
@@ -869,6 +888,7 @@ document.onkeydown = function (e) {
 		oldcfg = [];
 		ebi('files').innerHTML = orig_html;
 		orig_html = null;
+		msel.render();
 		reload_browser();
 	}
 })();
@@ -877,12 +897,16 @@ document.onkeydown = function (e) {
 var treectl = (function () {
 	var treectl = {
 		"hidden": false
-	};
-	var dyn = bcfg_get('dyntree', true);
-	var treesz = icfg_get('treesz', 16);
+	},
+		entreed = false,
+		fixedpos = false,
+		prev_atop = null,
+		prev_winh = null,
+		dyn = bcfg_get('dyntree', true),
+		treesz = icfg_get('treesz', 16);
+
 	treesz = Math.min(Math.max(treesz, 4), 50);
 	console.log('treesz [' + treesz + ']');
-	var entreed = false;
 
 	function entree(e) {
 		ev(e);
@@ -914,13 +938,43 @@ var treectl = (function () {
 		if (!entreed || treectl.hidden)
 			return;
 
-		var top = ebi('wrap').getBoundingClientRect().top;
-		ebi('tree').style.top = Math.max(0, parseInt(top)) + 'px';
+		var tree = ebi('tree'),
+			wrap = ebi('wrap'),
+			atop = wrap.getBoundingClientRect().top,
+			winh = window.innerHeight;
+
+		if (atop === prev_atop && winh === prev_winh)
+			return;
+
+		prev_atop = atop;
+		prev_winh = winh;
+
+		if (fixedpos && atop >= 0) {
+			tree.style.position = 'absolute';
+			tree.style.bottom = '';
+			fixedpos = false;
+		}
+		else if (!fixedpos && atop < 0) {
+			tree.style.position = 'fixed';
+			tree.style.height = 'auto';
+			fixedpos = true;
+		}
+
+		if (fixedpos) {
+			tree.style.top = Math.max(0, parseInt(atop)) + 'px';
+		}
+		else {
+			var top = Math.max(0, parseInt(wrap.offsetTop)),
+				treeh = (winh - atop) - 4;
+
+			tree.style.top = top + 'px';
+			tree.style.height = treeh < 10 ? '' : treeh + 'px';
+		}
 	}
 
 	function periodic() {
 		onscroll();
-		setTimeout(periodic, document.visibilityState ? 200 : 5000);
+		setTimeout(periodic, document.visibilityState ? 100 : 5000);
 	}
 	periodic();
 
@@ -997,8 +1051,6 @@ var treectl = (function () {
 					var o = links[a].parentNode;
 					if (!o.getElementsByTagName('li').length)
 						o.innerHTML = html;
-					//else
-					//	links[a].previousSibling.textContent = '-';
 				}
 			}
 		}
@@ -1282,7 +1334,7 @@ function find_file_col(txt) {
 
 function mk_files_header(taglist) {
 	var html = [
-		'<thead>',
+		'<thead><tr>',
 		'<th name="lead"><span>c</span></th>',
 		'<th name="href"><span>File Name</span></th>',
 		'<th name="sz" sort="int"><span>Size</span></th>'
@@ -1301,7 +1353,7 @@ function mk_files_header(taglist) {
 	html = html.concat([
 		'<th name="ext"><span>T</span></th>',
 		'<th name="ts"><span>Date</span></th>',
-		'</thead>',
+		'</tr></thead>',
 	]);
 	return html;
 }
@@ -1335,13 +1387,13 @@ var filecols = (function () {
 				continue;
 
 			var name = span[0].textContent,
-				cls = '';
+				cls = false;
 
 			if (has(hidden, name)) {
 				ohidden.push(a);
-				cls = ' min';
+				cls = true;
 			}
-			ths[a].className = ths[a].className.replace(/ *min */, " ") + cls;
+			clmod(ths[a], 'min', cls)
 		}
 		for (var a = 0; a < ncols; a++) {
 			var cls = has(ohidden, a) ? 'min' : '';
@@ -1512,12 +1564,15 @@ var mukey = (function () {
 
 function addcrc() {
 	var links = document.querySelectorAll(
-		'#files>tbody>tr>td:nth-child(2)>' + (
+		'#files>tbody>tr>td:first-child+td>' + (
 			ebi('unsearch') ? 'div>a:last-child' : 'a'));
 
 	for (var a = 0, aa = links.length; a < aa; a++)
-		if (!links[a].getAttribute('id'))
-			links[a].setAttribute('id', 'f-' + crc32(links[a].textContent));
+		if (!links[a].getAttribute('id')) {
+			var crc = crc32(links[a].textContent || links[a].innerText);
+			crc = ('00000000' + crc).slice(-8);
+			links[a].setAttribute('id', 'f-' + crc);
+		}
 }
 
 
@@ -1544,6 +1599,24 @@ function addcrc() {
 })();
 
 
+(function () {
+	var light = bcfg_get('lightmode', false);
+
+	function freshen() {
+		document.documentElement.setAttribute("class", light ? "light" : "");
+	}
+
+	ebi('lightmode').onclick = function (e) {
+		ev(e);
+		light = !light;
+		bcfg_set('lightmode', light);
+		freshen();
+	};
+
+	freshen();
+})();
+
+
 var arcfmt = (function () {
 	if (!ebi('arc_fmt'))
 		return { "render": function () { } };
@@ -1620,27 +1693,26 @@ var msel = (function () {
 		return names;
 	}
 	function selui() {
-		var fun = getsel().length ? "add" : "remove";
-		ebi('wtoggle').classList[fun]('sel');
+		clmod(ebi('wtoggle'), 'sel', getsel().length);
 	}
 	function seltgl(e) {
 		ev(e);
 		var tr = this.parentNode;
-		tr.classList.toggle('sel');
+		clmod(tr, 'sel', 't');
 		selui();
 	}
 	function evsel(e, fun) {
 		ev(e);
 		var trs = document.querySelectorAll('#files tbody tr');
 		for (var a = 0, aa = trs.length; a < aa; a++)
-			trs[a].classList[fun]('sel');
+			clmod(trs[a], 'sel', fun);
 		selui();
 	}
 	ebi('selall').onclick = function (e) {
 		evsel(e, "add");
 	};
 	ebi('selinv').onclick = function (e) {
-		evsel(e, "toggle");
+		evsel(e, "t");
 	};
 	ebi('selzip').onclick = function (e) {
 		ev(e);
